import 'package:cached_network_image/cached_network_image.dart';
import 'package:connect_flutter/service/softcorp_service.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:progress_state_button/iconed_button.dart';
import 'package:progress_state_button/progress_button.dart';
import 'package:softcorp_flutter_core/constants.dart';
import 'package:softcorp_flutter_core/default_form_field/default_form_field.dart';
import 'package:softcorp_flutter_core/default_spinner/default_spinner.dart';
import 'package:websafe_svg/websafe_svg.dart';
import 'package:dart_softcorp/softcorp.pbgrpc.dart' as SoftcorpGrpc;
import 'dart:html' as html;

class SignInScreen extends StatefulWidget {
  final Function changeScreen;
  final SoftcorpGrpc.ClientApp? app;
  final String loginChallenge;
  final bool? previewMode;

  const SignInScreen({
    Key? key,
    required this.changeScreen,
    required this.app,
    required this.loginChallenge,
    required this.previewMode,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _SignInScreenState();
  }
}

class _SignInScreenState extends State<SignInScreen> {
  final TextEditingController passwordController = TextEditingController();
  final TextEditingController emailController = TextEditingController();
  final _formKey = GlobalKey<FormState>();
  ButtonState buttonState = ButtonState.idle;

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.center,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        SizedBox(
          height: 60,
          child: CachedNetworkImage(
            height: 60,
            imageUrl: widget.app!.logoUrl,
            progressIndicatorBuilder: (context, url, downloadProgress) =>
                const Center(
              child: DefaultSpinner(
                backgroundColor: Colors.transparent,
                size: 60,
              ),
            ),
            errorWidget: (context, url, error) => WebsafeSvg.network(
              widget.app!.logoUrl,
            ),
          ),
        ),
        const SizedBox(height: defaultPadding * 2),
        Text(
          "Sign in to " + widget.app!.name,
          style: const TextStyle(
            color: defaultTitleColor,
            fontSize: 25,
            fontWeight: FontWeight.w400,
            overflow: TextOverflow.ellipsis,
          ),
          maxLines: 2,
        ),
        const SizedBox(height: defaultPadding),
        Form(
          key: _formKey,
          child: Column(
            children: [
              SizedBox(
                width: 500,
                child: DefaultFormField(
                  textEditingController: emailController,
                  textInputType: TextInputType.emailAddress,
                  hint: "Enter your email address",
                  label: "Email",
                  prefixIcon: Icons.email_rounded,
                  addDefaultBorder: true,
                  validator: (String? email) {
                    if (RegExp(
                            r"^[a-zA-Z0-9.a-zA-Z0-9.!#$%&'*+-/=?^_`{|}~]+@[a-zA-Z0-9]+\.[a-zA-Z]+")
                        .hasMatch(email ?? "")) {
                      return null;
                    }
                    return "Please enter a valid email";
                  },
                ),
              ),
              SizedBox(
                width: 500,
                child: DefaultFormField(
                  textEditingController: passwordController,
                  textInputType: TextInputType.text,
                  hint: "Enter your password",
                  prefixIcon: Icons.lock_rounded,
                  label: "Password",
                  addDefaultBorder: true,
                  obscureText: true,
                  validator: (String? password) {
                    if (password == null || password == "") {
                      return "Please enter a valid password";
                    }
                    return null;
                  },
                ),
              ),
            ],
          ),
        ),
        const SizedBox(height: defaultPadding * 2),
        ProgressButton.icon(
          height: 50,
          iconedButtons: const {
            ButtonState.idle: IconedButton(
                text: "Sign in",
                icon: Icon(Icons.login_rounded, color: Colors.white),
                color: Colors.black),
            ButtonState.loading:
                IconedButton(text: "Loading", color: defaultPrimaryColor),
            ButtonState.fail: IconedButton(
                text: "Invalid credentials",
                icon: Icon(Icons.cancel, color: Colors.white),
                color: defaultDangerColor),
            ButtonState.success: IconedButton(
                text: "Success",
                icon: Icon(
                  Icons.check_circle,
                  color: Colors.white,
                ),
                color: defaultSuccessColor)
          },
          onPressed: () {
            if (widget.previewMode != true &&
                _formKey.currentState!.validate() &&
                buttonState == ButtonState.idle) {
              setState(() {
                buttonState = ButtonState.loading;
              });
              SoftcorpService.validateUserCredentials(
                      loginChallenge: widget.loginChallenge,
                      email: emailController.text,
                      password: passwordController.text)
                  .catchError((onError) {
                print(onError);
                setState(() {
                  buttonState = ButtonState.fail;
                  emailController.text = "";
                  passwordController.text = "";
                });
                Future.delayed(const Duration(seconds: 2)).then((value) {
                  setState(() {
                    buttonState = ButtonState.idle;
                  });
                });
              }).then((value) {
                if (value.redirectAddr != "") {
                  setState(() {
                    buttonState = ButtonState.success;
                  });
                  Future.delayed(const Duration(seconds: 1)).then((_) {
                    html.window.location.href = value.redirectAddr;
                  });
                }
              });
            }
          },
          state: buttonState,
          radius: defaultCornerRadius,
        ),
        const SizedBox(height: 10),
        TextButton(
          onPressed: () {
            widget.changeScreen();
          },
          child: const Text(
            "Do you not have an account? Create one.",
            style: TextStyle(
              color: defaultPrimaryColor,
              fontWeight: FontWeight.w400,
            ),
          ),
        ),
        TextButton(
          onPressed: () {
            widget.changeScreen();
          },
          child: const Text(
            "Forgot your password?",
            style: TextStyle(
              color: defaultPrimaryColor,
              fontWeight: FontWeight.w400,
            ),
          ),
        ),
      ],
    );
  }
}
